==> libggi_1.99.2.0b2.1.orig.tar.gz: libggi-2.0b2.1/display/fbdev/color.c <==
/* $Id: color.c,v 1.20 1999/05/05 20:27:39 marcus Exp $
******************************************************************************

   Display-FBDEV

   Copyright (C) 1998 Andrew Apted    [andrew@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/fb.h>

#include <ggi/internal/ggi-dl.h>
#include <ggi/display/fbdev.h>


int
GGI_fbdev_setpalvec(ggi_visual *vis, int start, int len, ggi_color *colormap)
{
	fbdev_hook *ff = LIBGGI_PRIVATE(vis);
	struct fb_cmap cmap;
	int nocols = 1 << GT_DEPTH(LIBGGI_GT(vis));

	GGIDPRINT_COLOR("display-fbdev: SetPalVec(%d,%d)\n", start, len);
	
	if (start == GGI_PALETTE_DONTCARE) {
		start = 0;
	}

	if ((start < 0) || (len < 0) || (start+len > nocols)) {
		return -1;
	}

	memcpy(vis->palette+start, colormap, len*sizeof(ggi_color));

	if (!ff->ismapped) return 0;

	cmap.start  = start;
	cmap.len    = len;
	cmap.red    = & ff->reds[start];
	cmap.green  = & ff->greens[start];
	cmap.blue   = & ff->blues[start];
	cmap.transp = NULL;

	for (; len > 0; start++, colormap++, len--) {
		ff->reds[start]   = colormap->r;
		ff->greens[start] = colormap->g;
		ff->blues[start]  = colormap->b;
	}

	if (ioctl(LIBGGI_FD(vis), FBIOPUTCMAP, &cmap) < 0) {
		GGIDPRINT_COLOR("display-fbdev: PUTCMAP failed.");
		return -1;
	}

	return 0;
}

==> libggi-2.0b1.tar.gz: libggi-2.0b1/display/fbdev/color.c <==
/* $Id: color.c,v 1.17 1998/12/20 21:30:55 marcus Exp $
******************************************************************************

   Display-FBDEV

   Copyright (C) 1998 Andrew Apted    [andrew@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/fb.h>

#include <ggi/internal/ggi-dl.h>
#include <ggi/display/fbdev.h>


void _GGI_fbdev_save_palette(ggi_visual *vis)
{
	fbdev_hook *ff = LIBGGI_PRIVATE(vis);

	struct fb_cmap cmap;

	if ((ff->orig_var.bits_per_pixel < 1) || 
	    (ff->orig_var.bits_per_pixel > 8)) {
		return;
	}

	cmap.start  = 0;
	cmap.len    = 1 << ff->orig_var.bits_per_pixel;
	cmap.red    = ff->orig_reds;
	cmap.green  = ff->orig_greens;
	cmap.blue   = ff->orig_blues;
	cmap.transp = NULL;

	if (ioctl(LIBGGI_FD(vis), FBIOGETCMAP, &cmap) < 0) {
		DPRINT_COLOR("display-fbdev: GETCMAP failed.\n");
		return;
	}

	DPRINT_COLOR("display-fbdev: Saved palette (len=%d).\n", cmap.len);
}

void _GGI_fbdev_restore_palette(ggi_visual *vis)
{
	fbdev_hook *ff = LIBGGI_PRIVATE(vis);

	struct fb_cmap cmap;

	if ((ff->orig_var.bits_per_pixel < 1) || 
	    (ff->orig_var.bits_per_pixel > 8)) {
		return;
	}

	cmap.start  = 0;
	cmap.len    = 1 << ff->orig_var.bits_per_pixel;
	cmap.red    = ff->orig_reds;
	cmap.green  = ff->orig_greens;
	cmap.blue   = ff->orig_blues;
	cmap.transp = NULL;

	if (ioctl(LIBGGI_FD(vis), FBIOPUTCMAP, &cmap) < 0) {
		DPRINT_COLOR("display-fbdev: PUTCMAP failed.\n");
		return;
	}

	DPRINT_COLOR("display-fbdev: Restored palette (len=%d).\n", 
			cmap.len);
}


/* ---------------------------------------------------------------------- */


int GGI_fbdev_setpalvec(ggi_visual *vis, int start, int len, ggi_color *colormap)
{
	fbdev_hook *ff = LIBGGI_PRIVATE(vis);

	struct fb_cmap cmap;

	int nocols = 1 << GT_DEPTH(LIBGGI_GT(vis));


	DPRINT_COLOR("display-fbdev: SetPalVec(%d,%d)\n", start, len);
	
	if (start == GGI_PALETTE_DONTCARE) {
		start = 0;
	}

	if ((start < 0) || (len < 0) || (start+len > nocols)) {
		return -1;
	}

	memcpy(vis->palette+start, colormap, len*sizeof(ggi_color));

	cmap.start  = start;
	cmap.len    = len;
	cmap.red    = & ff->reds[start];
	cmap.green  = & ff->greens[start];
	cmap.blue   = & ff->blues[start];
	cmap.transp = NULL;

	for (; len > 0; start++, colormap++, len--) {
	
		ff->reds[start]   = colormap->r;
		ff->greens[start] = colormap->g;
		ff->blues[start]  = colormap->b;
	}

	if (ioctl(LIBGGI_FD(vis), FBIOPUTCMAP, &cmap) < 0) {
		DPRINT_COLOR("display-fbdev: PUTCMAP failed.");
		return -1;
	}

	return 0;
}
